package mv3302;

import simkit.*;
import java.util.*;
import simkit.random.RandomVariate;

public class TransferLineComponent extends SimEntityBase {

    private int[] totalNumberMachines;
    private RandomVariate[] processingTimeGenerator;
    protected int[] numberAvailableMachines;
    protected List<SortedSet<Part>> queue;
    protected double[] delayInQueue;
    protected double[] timeAtStation;
    protected double totalDelayInQueue;
    protected double totalTimeInSystem;
    protected int numberComplete;

    /**
     * Constructor method for TransferLineComponent that passes in Array values
     * of processingTimeGenerator and totalNumberMachines as sets them
     * accordingly. Sets the values of numberAvailableMachines to
     * totalNumberMachines. Sets the length of the delayInQueue and
     * timeAtStation arrays to the length of totalNumberMachines. Instantiates
     * the value of the queue to an empty array with elements as treesets the
     * length of elements in totalNumberMachines
     *
     * @param processingTimeGenerator
     * @param totalNumberMachines
     */
    public TransferLineComponent(RandomVariate[] processingTimeGenerator, int[] totalNumberMachines) {

        this();
        this.setProcessingTimeGenerator(processingTimeGenerator);
        this.setTotalNumberMachines(totalNumberMachines);
        numberAvailableMachines = totalNumberMachines.clone();
        delayInQueue = new double[totalNumberMachines.length];
        timeAtStation = new double[totalNumberMachines.length];
        queue = new ArrayList<>();
        for (int station = 0; station < totalNumberMachines.length; ++station) {
            queue.add(new TreeSet<>());
        }
    }

    /**
     * Zero Argument constructor method that makes a call to it's super.
     *
     */
    public TransferLineComponent() {
        super();

    }

    /**
     * Reset method resets all values to their base before the start of the next
     * machine run. resets the numberOfAvailable machines back to the same
     * property as the totalNumberMachines. Also resets the values contained
     * within the delayInQueue and timeAtStation array back to the default NaN
     * value. Clears the contents of the queue.
     */
    @Override
    public void reset() {
        super.reset();
        numberAvailableMachines = getTotalNumberMachines().clone();
        Arrays.fill(delayInQueue, Double.NaN);
        Arrays.fill(timeAtStation, Double.NaN);

    }

    /**
     * Do run method instantiates the total delay in queue and time in system
     * variables to NaN value. Fires Property changes for total delay in queue,
     * total delay in system and number of available machines. Schedules next
     * Init Event.
     */
    public void doRun() {
        totalDelayInQueue = Double.NaN;
        totalTimeInSystem = Double.NaN;
//        firePropertyChange("totalDelayInQueue", getTotalDelayInQueue());
//        firePropertyChange("totalDelayInSystem", getTotalTimeInSystem());
//        firePropertyChange("numberAvailableMachines", getNumberAvailableMachines());
        waitDelay("Init", 0.0, 0);
    }

    /**
     * Init method starts with 0 as the parameter and aligns the indexes for the
     * number available machines to the indexes of totalNumberMachines.Fires the
     * necessary index property changes. Calls the queue array for the element
     * at index i and clears that element. It then sets the values of the
     * delayInQueue and timeAtStation array elements back to NaN. It then
     * schedules another Init event as long as the i argument is less than the
     * totalNumberOfMachines length minus one to account for the off by one 0
     * index.
     *
     * @param i
     */
    public void doInit(int i) {
        this.numberAvailableMachines[i] = totalNumberMachines[i];
//            fireIndexedPropertyChange(i, "numberAvailableMachines", getNumberAvailableMachines(i));
        queue.get(i).clear();
//            fireIndexedPropertyChange(i, "queue", getQueue(i));
//            double[] oldDelayInQueue = getDelayInQueue().clone();
        delayInQueue[i] = Double.NaN;
//            firePropertyChange("delayInQueue", oldDelayInQueue, getDelayInQueue());
//            double[] oldTimeAtStation = getTimeAtStation().clone();
        timeAtStation[i] = Double.NaN;
//            firePropertyChange("timeAtStation", oldTimeAtStation, getTimeAtStation());
        if (i < totalNumberMachines.length - 1) {
            waitDelay("Init", 0.0, i + 1);
        }
    }

    /**
     * doArrival() method takes in arguments Part Part, and int station.The
     * method takes a time stamp on the part, Method then takes the queue at the
     * station i and adds the part value to it. Fires changes to the listener
     * then schedules the startProcessing event if numberAvailableMachines is
     * greater than 0.
     *
     * @param part
     * @param station
     */
    public void doArrival(Part part, int station) {
        part.stampTime();
        SortedSet<Part> oldQueue = getQueue(station);
        queue.get(station).add(part);
        fireIndexedPropertyChange(station, "queue", oldQueue, getQueue(station));
        if (numberAvailableMachines[station] > 0) {
            waitDelay("StartProcessing", 0.0, station);
        }
    }

    /**
     * doStartProcessing method takes the part at station and removes it from
     * the queue, The number of available machines at station are reduced by
     * one. the delay in queue array value at station is set to the part elapsed
     * time. The part delay in queue is then incremented
     *
     * @param station
     */
    public void doStartProcessing(int station) {
        SortedSet<Part> oldQueue = queue.get(station);
        Part part = oldQueue.first();

        queue.get(station).remove(part);
//fireIndexedPropertyChange(station, "queue", oldQueue, getQueue(station));
        int[] oldNumberAvailableMachines = getNumberAvailableMachines().clone();
        numberAvailableMachines[station]--;
        fireIndexedPropertyChange(station, "numberAvailableMachines", oldNumberAvailableMachines[station], getNumberAvailableMachines()[station]);
        double[] oldDelayInQueue = getDelayInQueue();
        delayInQueue[station] = part.getElapsedTime();

        fireIndexedPropertyChange(station, "delayInQueue", oldDelayInQueue[station], delayInQueue[station]);
        part.incrementDelayInQueue(delayInQueue[station]);

        waitDelay("EndProcessing", getProcessingTimeGenerator()[station].generate(), part, station);
    }

    /**
     * doEndProcessing() method takes in arguments for part and i. Takes
     * numberAvailableMachines at element i and increases it by one. Also
     * assigns timeAtStation to the elapsed time variable of the current part.
     * Finally determines the path for scheduling based on the preferred
     * conditions.
     */
    public void doEndProcessing(Part part, int station) {
        int[] oldNumberAvailableMachines = getNumberAvailableMachines().clone();
        numberAvailableMachines[station]++;
        fireIndexedPropertyChange(station, "numberAvailableMachines", oldNumberAvailableMachines[station], getNumberAvailableMachines()[station]);
        double[] oldTimeAtStation = getTimeAtStation().clone();
        timeAtStation[station] = part.getElapsedTime();
        fireIndexedPropertyChange(station, "timeAtStation", oldTimeAtStation[station], getTimeAtStation(station));
        if (station < totalNumberMachines.length - 1) {
            waitDelay("Arrival", 0.0, part, station + 1);
        }
        if (station == totalNumberMachines.length - 1) {
            waitDelay("PartComplete", 0.0, part);
        }
        if (!queue.get(station).isEmpty()) {
            waitDelay("StartProcessing", 0.0, station);
        }
    }

    /**
     * doPartComplete brings in argument part and assigns the value of
     * totalTimeInSystem to the parts getAge() value. It also assigns the
     * totalDelayInQueue variable to the part's totalDelay(). Increments the
     * number of completed parts by one.
     */
    public void doPartComplete(Part part) {
        double oldTotalTimeInSystem = totalTimeInSystem;
        totalTimeInSystem = part.getAge();
        firePropertyChange("totalTimeInSystem", oldTotalTimeInSystem, getTotalTimeInSystem());
        double oldTotalDelayInQueue = totalDelayInQueue;
        totalDelayInQueue = part.getTotalDelayInQueue();
        firePropertyChange("totalDelayInQueue", oldTotalDelayInQueue, getTotalDelayInQueue());
    }

    /**
     * @return the totalNumberMachines
     */
    public int[] getTotalNumberMachines() {
        return totalNumberMachines.clone();
    }

    /**
     * @param station
     * @return the totalNumberMachines @ station
     */
    public int getTotalNumberMachines(int station) {
        return totalNumberMachines[station];
    }

    /**
     * @return the processingTimeGenerator
     */
    public RandomVariate[] getProcessingTimeGenerator() {
        return processingTimeGenerator.clone();
    }

    /**
     * @return the processingTimeGenerator @ station
     */
    public RandomVariate getProcessingTimeGenerator(int station) {
        return processingTimeGenerator[station];
    }

    /**
     * @return the numberAvailableMachines
     */
    public int[] getNumberAvailableMachines() {
        return numberAvailableMachines.clone();
    }

    /**
     * @param station
     * @return the numberAvailableMachines @ station
     */
    public int getNumberAvailableMachines(int station) {
        return numberAvailableMachines[station];
    }

    /**
     * @param station
     * @return the queue @ station
     */
    public SortedSet<Part> getQueue(int station) {
        return new TreeSet<>(queue.get(station));
    }

    /**
     * @return the queue
     */
    public List<SortedSet<Part>> getQueue() {
        List<SortedSet<Part>> queueListCopy = new ArrayList<>();
        for (int station = 0; station < queue.size(); ++station) {
            queueListCopy.add(getQueue(station));
        }
        return queueListCopy;
    }

    /**
     * @return the delayInQueue
     */
    public double[] getDelayInQueue() {
        return delayInQueue.clone();
    }

    /**
     * @param station
     * @return the delayInQueue @ station
     */
    public double getDelayInQueue(int station) {
        return delayInQueue[station];
    }

    /**
     * @return the timeAtStation
     */
    public double[] getTimeAtStation() {
        return timeAtStation.clone();
    }

    /**
     * @param station
     * @return the timeAtStation @ station
     */
    public double getTimeAtStation(int station) {
        return timeAtStation[station];
    }

    /**
     * @return the totalDelayInQueue
     */
    public double getTotalDelayInQueue() {
        return totalDelayInQueue;
    }

    /**
     * @return the totalTimeInSystem
     */
    public double getTotalTimeInSystem() {
        return totalTimeInSystem;
    }

    /**
     * sets TotalNumberMachines variable
     *
     * @param totalNumberMachines
     */
    public void setTotalNumberMachines(int[] totalNumberMachines) {
        this.totalNumberMachines = totalNumberMachines;
    }

    /**
     * sets processingTimeGenerator variable
     *
     * @param processingTimeGenerator
     */
    public void setProcessingTimeGenerator(RandomVariate[] processingTimeGenerator) {
        this.processingTimeGenerator = processingTimeGenerator;
    }

    /**
     * @return the numberComplete
     */
    public int getNumberComplete() {
        return numberComplete;
    }

}
