package mv3302.run;

import static java.lang.Math.sqrt;
import mv3302.PartArrivalProcess;
import mv3302.StoppedPartArrivalProcess;
import mv3302.TransferLineComponent;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.stat.SimpleStatsTally;
import simkit.stat.StudentT;

/**
 *
 * @author dansl
 */
public class RunTransientAnalysis {

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        // Create an exponential random variate generator with a mean of 2.5
        RandomVariate interarrivalTimeGenerator = RandomVariateFactory.getInstance(
                "Exponential", 2.5);
        // Define the stop time for the simulation (in minutes)
        int stopTime = 480;
        // Create a part arrival process using the interarrival time generator
        PartArrivalProcess partArrivalProcess = new PartArrivalProcess(interarrivalTimeGenerator);
        // Create a stopped part arrival process with the given stop time
        StoppedPartArrivalProcess stoppedArrivalProcess = new StoppedPartArrivalProcess(stopTime);
        // Create an array of random variate generators for processing times
        RandomVariate[] processingTimeGenerator
                = {RandomVariateFactory.getInstance("Exponential", 2.4), 
                    RandomVariateFactory.getInstance("Gamma", 3.2, 3.3), 
                    RandomVariateFactory.getInstance("Uniform", 4.5, 6.7), 
                    RandomVariateFactory.getInstance("Exponential", 3.0)};
        // Define the total number of workstations for each type of processing time
        int[] totalNumberWorkstations = new int[4];
        totalNumberWorkstations[0] = 1;
        totalNumberWorkstations[1] = 5;
        totalNumberWorkstations[2] = 4;
        totalNumberWorkstations[3] = 2;
        // Create a transfer line component with the processing time generators and workstation numbers
        TransferLineComponent transferLineComponent = new TransferLineComponent(processingTimeGenerator, totalNumberWorkstations);
        // Add the transfer line component as a listener to the part arrival processes
        partArrivalProcess.addSimEventListener(transferLineComponent);
        stoppedArrivalProcess.addSimEventListener(transferLineComponent);
        // Create a statistics tally for total time in the system
        SimpleStatsTally totalTimeInSystemStat = new SimpleStatsTally("totalTimeInSystem");
        // Add the statistics tally as a property change listener to the transfer line component
        transferLineComponent.addPropertyChangeListener(totalTimeInSystemStat);

        System.out.println(partArrivalProcess);
        System.out.println(stoppedArrivalProcess);
        System.out.println(transferLineComponent);
        // Create a statistics tally for outer time in the system
        SimpleStatsTally outerTimeInSystemStat = new SimpleStatsTally("outerTimeInSystem");
        // Define the confidence level (alpha)
        double alpha = 0.05;
        // Stop the simulation after 10,000 events with the name "Arrival"
        Schedule.stopOnEvent(10000, "Arrival");

        System.out.println("Avg Delay In Queue");
        // Run the replications
        for (int replication = 1; replication <= stopTime; ++replication) {
            Schedule.reset();
            // Reset the statistics tally for total time in the system
            totalTimeInSystemStat.reset();
            Schedule.startSimulation();
            // Record the outer time in the system using the mean of the total time in the system
            outerTimeInSystemStat.newObservation(totalTimeInSystemStat.getMean());
            // Print the replication number, mean, and confidence interval
            System.out.printf("%d %.3f ± %.3f%n",
                    replication,
                    outerTimeInSystemStat.getMean(),
                    outerTimeInSystemStat.getStandardDeviation()
                    * StudentT.getQuantile(1.0 - 0.5 * alpha, outerTimeInSystemStat.getCount() - 1) / sqrt(outerTimeInSystemStat.getCount())
            );
        }
    }
}
